/* fperror:  check to see if there has been a floating point error by
             testing the summary bit in the FPU status word. If so, print
             an error message listing all the bits that are set, and set errno
             accordingly. Finally, a SIGFPE signal is sent to all processes
             in the calling process' execution group */

#include<stdio.h>
#include<errno.h>
#include<sys/types.h>
#include<signal.h>
extern int _fpucw, _fpusw; 
extern void _store_fpu_state();

#define INVALIDOP 0x0001
#define DENORM    0x0002
#define DIVBYZERO 0x0004
#define OVERFLOW  0x0008
#define UNDERFLOW 0x0010
#define PRECISION 0x0010
#define SUMMARY   0x0040

static void whine(int);
static void set_errno(int);

/* Levels of Agression */
#define MILD 0
#define MODERATE 1
#define SEVERE 2
static int lvl = MILD;

int _fperror(int paranoia)
{
	lvl = paranoia;
	_store_fpu_state();
	if(_fpusw & SUMMARY){
		if(lvl == MILD){
			 set_errno(_fpusw);
			 return 1;
		}
		if((lvl == SEVERE)||(lvl == MODERATE))whine(_fpusw);
		if(lvl == SEVERE)
			if((_fpusw && DIVBYZERO)||(_fpusw && OVERFLOW))
				kill(getpid(),SIGFPE);
		return 1;
	}
	return 0;
}

static void whine(int x){

	if(lvl == SEVERE) {
	   if(x & UNDERFLOW)fprintf(stderr," Underflow \n");
	   if(x & DENORM)fprintf(stderr," Denormalized Operand \n");
	   if(x & PRECISION)fprintf(stderr," Precision \n");
	  }

	if((x & DIVBYZERO)||(x & OVERFLOW)){
		if(x & DIVBYZERO) fprintf(stderr," Zero Divide \n");
		if(x & OVERFLOW) fprintf(stderr," Overflow \n");
	}
	return;
}

static void set_errno(int x){

	if(x & DIVBYZERO)errno = EDOM;
	if(x & OVERFLOW)errno = ERANGE;
	if(x & UNDERFLOW)errno = ERANGE;
	if(x & PRECISION)errno = ERANGE;

}
