# This is the C run-time start-off routine.  Its job is to take the
# arguments as put on the stack by EXEC, and to parse them and set them up the
# way main expects them.
# It also initializes environ when this variable isn't defined by the
# programmer.  The detection of whether environ belongs to us is rather
# simplistic.  We simply check for some magic value, but there is no other
# way.

.globl crtso, __main, _penviron, _penvp, _fpu_present
.text

crtso:
	xor     %ebp, %ebp		# clear for backtrace of core files
	mov     (%esp), %eax    	# argc
	lea     4(%esp), %edx		# argv
	lea     8(%esp,%eax,4), %ecx	# envp    disp(base,index,scale)

	# Test if environ is in the initialized data area and is set to our
	# magic number.  If so then it is not redefined by the user.
	# First store the address of environ in ebx. (environ is defined
	# and initialized to contain 5353... in other/environ.c)

	lea	environ, %ebx           # Address of environ in ebx
	cmp	_edata, %ebx	        # within initialized data?
	jae	_setenvp                # jump, if not. (next 0: forward.)
	testb	 $3, %bl        	# aligned?
	jnz	_setenvp                # jump, if not
	cmpl	$0x53535353,(%ebx)	# is it our _environ?
	jne	_setenvp                # jump, if not (local symbol)

	# _penviron is a char *** which contains the ADDRESS of the
	# pointer to the environment block. It is used by the library,
	# esp. putenv. It is hidden from the user by virtue of not being
	# documented (though since you're reading this it's obviously
	# not hidden from you, har, har.) 

	mov	%ebx, _penviron		# _penviron = &environ;
	mov	_penviron, %ebx       	# ebx = &environ
_setenvp:
	mov	%ecx,(%ebx)	        # environ = envp;

	push	%ecx			# push envp
	push	%edx			# push argv
	push	%eax			# push argc

	# Test the EM bit of the MSW to determine if an FPU is present and
	# set _fpu_present if one is found.
	smsw	%ax                     # store machine status word
	testb	$0x4, %al		# EM bit in MSW
	setz	(_fpu_present)		# True if not set
	finit                           # initialize floating point unit
	call	main			# main(argc, argv, envp)

	push	%eax			# push exit status
	call	exit

	hlt				# force a trap if exit fails

.data
	.long 0				# Separate I&D: *NULL == 0
_penviron:
	.long _penvp			# Pointer to environ, or hidden pointer

.bss
					# Not sure what this is for.
	.comm	_penvp, 4		# Hidden environment vector
	.comm	_fpu_present, 4	        # FPU present flag
	.comm	_msw,4
