#
# double frexp(double r, int *exp):  extract exponent of r into
# integer pointed to, return mantissa of r
#
# According to libc definition, this returns a double in [1/2,1)
# and a power of 2 through exp. If r = 0 then both *exp and the
# returned value are 0. 
#
# A small difficulty is that the floating point 
# unit seems to give r in [1,2). This must be the IEEE standard.
# So before storing values for return we divide by 2 and increment exp
#
.text; .data; .data; .bss
.globl	frexp
.align 8
.text
frexp:
				  # This is AT&T syntax for gnu as
	pushl	%ebp              # save orig value of ebp
	movl    %esp, %ebp        # ebp now base of stack pointer 
	movl    16(%ebp),%edx     # eax now has exp (an address)
				  # save unnecessary regs, just to be nice
        pushl	%eax
        pushl	%edx
	                          # begin to pester FPU
	fldl    8(%ebp)           # push value of r on st(0)
	fxtract                   # extract exponent, mantissa to st(1) st(0)
	fxch    %st(1)            # now mantissa, exponent in st(1) st(0)
	fistpl  (%edx)            # store as int through pointer 
				  # renormalize to library format
	addl    $1,(%edx)
	fdivl   cupla
	ftst                      # sigh. This 3 step dance is because you cant
	fstsw                     # branch on an fpu condition. Store sw in eax
	sahf                      # and save relevant flags to cpu sw
	jnz    slan               # r=0 case: define exp = 0 here
	movl   $0, (%edx)
	                          # real value returned via st(0) 
slan:
	popl	%edx
	popl 	%eax
	leave	                  # sicut erat in principio
	ret

.data
cupla:   # This is the quad version of 2 in IEEE format:
	.long 0
	.long 1073741824
